@@ -48,10 +48,45 @@ std::unordered_map<KeySym, const char *, EnumHash> makeLookupKeyNameMap() {
         {FcitxKey_Up, NC_("Key name", "Up")},
         {FcitxKey_Right, NC_("Key name", "Right")},
         {FcitxKey_Down, NC_("Key name", "Down")},
-        {FcitxKey_Page_Up, NC_("Key name", "PgUp")},
-        {FcitxKey_Page_Down, NC_("Key name", "PgDown")},
+        {FcitxKey_Page_Up, NC_("Key name", "Page Up")},
+        {FcitxKey_Page_Down, NC_("Key name", "Page Down")},
         {FcitxKey_Caps_Lock, NC_("Key name", "CapsLock")},
         {FcitxKey_Num_Lock, NC_("Key name", "NumLock")},
+        {FcitxKey_KP_Space, NC_("Key name", "Keypad Space")},
+        {FcitxKey_KP_Tab, NC_("Key name", "Keypad Tab")},
+        {FcitxKey_KP_Enter, NC_("Key name", "Keypad Enter")},
+        {FcitxKey_KP_F1, NC_("Key name", "Keypad F1")},
+        {FcitxKey_KP_F2, NC_("Key name", "Keypad F2")},
+        {FcitxKey_KP_F3, NC_("Key name", "Keypad F3")},
+        {FcitxKey_KP_F4, NC_("Key name", "Keypad F4")},
+        {FcitxKey_KP_Home, NC_("Key name", "Keypad Home")},
+        {FcitxKey_KP_Left, NC_("Key name", "Keypad Left")},
+        {FcitxKey_KP_Up, NC_("Key name", "Keypad Up")},
+        {FcitxKey_KP_Right, NC_("Key name", "Keypad Right")},
+        {FcitxKey_KP_Down, NC_("Key name", "Keypad Down")},
+        {FcitxKey_KP_Page_Up, NC_("Key name", "Keypad Page Up")},
+        {FcitxKey_KP_Page_Down, NC_("Key name", "Keypad Page Down")},
+        {FcitxKey_KP_End, NC_("Key name", "Keypad End")},
+        {FcitxKey_KP_Begin, NC_("Key name", "Keypad Begin")},
+        {FcitxKey_KP_Insert, NC_("Key name", "Keypad Insert")},
+        {FcitxKey_KP_Delete, NC_("Key name", "Keypad Delete")},
+        {FcitxKey_KP_Equal, NC_("Key name", "Keypad =")},
+        {FcitxKey_KP_Multiply, NC_("Key name", "Keypad *")},
+        {FcitxKey_KP_Add, NC_("Key name", "Keypad +")},
+        {FcitxKey_KP_Separator, NC_("Key name", "Keypad ,")},
+        {FcitxKey_KP_Subtract, NC_("Key name", "Keypad -")},
+        {FcitxKey_KP_Decimal, NC_("Key name", "Keypad .")},
+        {FcitxKey_KP_Divide, NC_("Key name", "Keypad /")},
+        {FcitxKey_KP_0, NC_("Key name", "Keypad 0")},
+        {FcitxKey_KP_1, NC_("Key name", "Keypad 1")},
+        {FcitxKey_KP_2, NC_("Key name", "Keypad 2")},
+        {FcitxKey_KP_3, NC_("Key name", "Keypad 3")},
+        {FcitxKey_KP_4, NC_("Key name", "Keypad 4")},
+        {FcitxKey_KP_5, NC_("Key name", "Keypad 5")},
+        {FcitxKey_KP_6, NC_("Key name", "Keypad 6")},
+        {FcitxKey_KP_7, NC_("Key name", "Keypad 7")},
+        {FcitxKey_KP_8, NC_("Key name", "Keypad 8")},
+        {FcitxKey_KP_9, NC_("Key name", "Keypad 9")},
         {FcitxKey_Scroll_Lock, NC_("Key name", "ScrollLock")},
         {FcitxKey_Menu, NC_("Key name", "Menu")},
         {FcitxKey_Help, NC_("Key name", "Help")},
