@@ -58,7 +58,7 @@ enum class InputMethodMode { PhysicalKeyboard, OnScreenKeyboard };
  * Type of input method events.
  */
 enum class EventType : uint32_t {
-    EventTypeFlag = 0xffff0000,
+    EventTypeFlag = 0xfffff000,
     UserTypeFlag = 0xffff0000,
     // send by frontend, captured by core, input method, or module
     InputContextEventFlag = 0x0001000,
@@ -240,7 +240,8 @@ public:
      */
     bool isInputContextEvent() const {
         auto flag = static_cast<uint32_t>(EventType::InputContextEventFlag);
-        return (static_cast<uint32_t>(type_) & flag) == flag;
+        auto mask = static_cast<uint32_t>(EventType::EventTypeFlag);
+        return (static_cast<uint32_t>(type_) & mask) == flag;
     }
 
 protected:
