@@ -9,6 +9,7 @@
 
 #include <cstdlib>
 #include <fstream>
+#include <functional>
 #include <string>
 #include <type_traits>
 #include "fcitx-utils/charutils.h"
@@ -42,14 +43,18 @@ parseLayout(const std::string &layout) {
     return {layout.substr(0, pos), layout.substr(pos + 1)};
 }
 
-static inline const CandidateWord *
-nthCandidateIgnorePlaceholder(const CandidateList &candidateList, int idx) {
+using GetCandidateListSizeCallback = std::function<int()>;
+using GetCandidateWordCallback = std::function<const CandidateWord &(int idx)>;
+static inline const CandidateWord *nthCandidateIgnorePlaceholder(
+    GetCandidateListSizeCallback getCandidateListSizeCallback,
+    GetCandidateWordCallback getCandidateWordCallback, int idx) {
     int total = 0;
-    if (idx < 0 || idx >= candidateList.size()) {
+    const int size = getCandidateListSizeCallback();
+    if (idx < 0 || idx >= size) {
         return nullptr;
     }
-    for (int i = 0, e = candidateList.size(); i < e; i++) {
-        const auto &candidate = candidateList.candidate(i);
+    for (int i = 0; i < size; i++) {
+        const auto &candidate = getCandidateWordCallback(i);
         if (candidate.isPlaceHolder()) {
             continue;
         }
@@ -61,6 +66,28 @@ nthCandidateIgnorePlaceholder(const CandidateList &candidateList, int idx) {
     return nullptr;
 }
 
+static inline const CandidateWord *
+nthCandidateIgnorePlaceholder(const CandidateList &candidateList, int idx) {
+    return nthCandidateIgnorePlaceholder(
+        [&candidateList]() { return candidateList.size(); },
+        [&candidateList](int idx) -> const CandidateWord & {
+            return candidateList.candidate(idx);
+        },
+        idx);
+}
+
+static inline const CandidateWord *
+nthBulkCandidateIgnorePlaceholder(const BulkCandidateList &bulkCandidateList,
+                                  int idx) {
+
+    return nthCandidateIgnorePlaceholder(
+        [&bulkCandidateList]() { return bulkCandidateList.totalSize(); },
+        [&bulkCandidateList](int idx) -> const CandidateWord & {
+            return bulkCandidateList.candidateFromAll(idx);
+        },
+        idx);
+}
+
 static inline std::string readFileContent(const std::string &file) {
     std::ifstream fin(file, std::ios::binary | std::ios::in);
     std::vector<char> buffer;
@@ -202,6 +229,14 @@ static inline std::string stripLanguage(const std::string &lc) {
     return lang;
 }
 
+static inline bool isSingleModifier(const Key &key) {
+    return key.isModifier() && (key.states() == 0 ||
+                                key.states() == Key::keySymToStates(key.sym()));
+}
+static inline bool isSingleKey(const Key &key) {
+    return isSingleModifier(key) || !key.hasModifier();
+}
+
 } // namespace fcitx
 
 FCITX_DECLARE_LOG_CATEGORY(keyTrace);
