@@ -7,6 +7,7 @@
 
 #include "userinterfacemanager.h"
 #include <set>
+#include "fcitx/event.h"
 #include "action.h"
 #include "inputcontext.h"
 #include "instance.h"
@@ -70,6 +71,11 @@ public:
         action->setId(0);
     }
 
+    void updateDispatch(UserInterfaceComponent comp, InputContext *ic);
+
+    template <UserInterfaceComponent component>
+    void updateSingleComponent(InputContext *ic);
+
     UserInterface *ui_ = nullptr;
     std::string uiName_;
     std::vector<std::string> uis_;
@@ -86,8 +92,48 @@ public:
     AddonManager *addonManager_;
 
     IdAllocator ids_;
+    bool isVirtualKeyboardVisible_ = false;
 };
 
+template <>
+void UserInterfaceManagerPrivate::updateSingleComponent<
+    UserInterfaceComponent::InputPanel>(InputContext *ic) {
+    if (ui_ != nullptr && ui_->addonInfo() != nullptr &&
+        ui_->addonInfo()->uiType() == UIType::OnScreenKeyboard) {
+        ui_->update(UserInterfaceComponent::InputPanel, ic);
+    } else if (const auto &callback =
+                   ic->inputPanel().customInputPanelCallback()) {
+        callback(ic);
+    } else if (ic->capabilityFlags().test(
+                   CapabilityFlag::ClientSideInputPanel)) {
+        ic->updateClientSideUIImpl();
+    } else if (ui_) {
+        ui_->update(UserInterfaceComponent::InputPanel, ic);
+    }
+}
+
+template <>
+void UserInterfaceManagerPrivate::updateSingleComponent<
+    UserInterfaceComponent::StatusArea>(InputContext *ic) {
+    if (ui_) {
+        ui_->update(UserInterfaceComponent::StatusArea, ic);
+    }
+}
+
+void UserInterfaceManagerPrivate::updateDispatch(UserInterfaceComponent comp,
+                                                 InputContext *ic) {
+#define _UI_CASE(COMP)                                                         \
+    case COMP:                                                                 \
+        updateSingleComponent<COMP>(ic);                                       \
+        break
+
+    switch (comp) {
+        _UI_CASE(UserInterfaceComponent::InputPanel);
+        _UI_CASE(UserInterfaceComponent::StatusArea);
+    }
+#undef _UI_CASE
+}
+
 UserInterfaceManager::UserInterfaceManager(AddonManager *addonManager)
     : d_ptr(std::make_unique<UserInterfaceManagerPrivate>(addonManager)) {}
 
@@ -207,29 +253,37 @@ void UserInterfaceManager::flush() {
     FCITX_D();
     for (auto &p : d->updateList_) {
         for (auto comp : p.second) {
-            if (comp == UserInterfaceComponent::InputPanel &&
-                p.first->capabilityFlags().test(
-                    CapabilityFlag::ClientSideInputPanel)) {
-                p.first->updateClientSideUIImpl();
-            } else if (d->ui_) {
-                d->ui_->update(comp, p.first);
-            }
+            d->updateDispatch(comp, p.first);
         }
     }
     d->updateIndex_.clear();
     d->updateList_.clear();
 }
 
+bool isUserInterfaceValid(UserInterface *userInterface,
+                          InputMethodMode inputMethodMode) {
+    if (userInterface == nullptr || !userInterface->available() ||
+        userInterface->addonInfo() == nullptr) {
+        return false;
+    }
+
+    return (userInterface->addonInfo()->uiType() == UIType::OnScreenKeyboard &&
+            inputMethodMode == InputMethodMode::OnScreenKeyboard) ||
+           (userInterface->addonInfo()->uiType() == UIType::PhyscialKeyboard &&
+            inputMethodMode == InputMethodMode::PhysicalKeyboard);
+}
+
 void UserInterfaceManager::updateAvailability() {
     FCITX_D();
+    auto *instance = d->addonManager_->instance();
     auto *oldUI = d->ui_;
     UserInterface *newUI = nullptr;
     std::string newUIName;
     for (auto &name : d->uis_) {
         auto *ui =
             static_cast<UserInterface *>(d->addonManager_->addon(name, true));
-        if (ui && ui->available()) {
-            newUI = static_cast<UserInterface *>(ui);
+        if (isUserInterfaceValid(ui, instance->inputMethodMode())) {
+            newUI = ui;
             newUIName = name;
             break;
         }
@@ -248,10 +302,66 @@ void UserInterfaceManager::updateAvailability() {
             d->addonManager_->instance()->postEvent(UIChangedEvent());
         }
     }
+
+    updateVirtualKeyboardVisibility();
 }
 
 std::string UserInterfaceManager::currentUI() const {
     FCITX_D();
     return d->uiName_;
 }
+
+bool UserInterfaceManager::isVirtualKeyboardVisible() const {
+    FCITX_D();
+    return d->isVirtualKeyboardVisible_;
+}
+
+void UserInterfaceManager::showVirtualKeyboard() const {
+    FCITX_D();
+
+    auto *ui = d->ui_;
+    if (ui == nullptr || ui->addonInfo() == nullptr ||
+        ui->addonInfo()->uiType() != UIType::OnScreenKeyboard) {
+        return;
+    }
+
+    auto *vkui = static_cast<VirtualKeyboardUserInterface *>(ui);
+    vkui->showVirtualKeyboard();
+}
+
+void UserInterfaceManager::hideVirtualKeyboard() const {
+    FCITX_D();
+
+    auto *ui = d->ui_;
+    if (ui == nullptr || ui->addonInfo() == nullptr ||
+        ui->addonInfo()->uiType() != UIType::OnScreenKeyboard) {
+        return;
+    }
+
+    auto *vkui = static_cast<VirtualKeyboardUserInterface *>(ui);
+    vkui->hideVirtualKeyboard();
+}
+
+void UserInterfaceManager::updateVirtualKeyboardVisibility() {
+    FCITX_D();
+    bool oldVisible = d->isVirtualKeyboardVisible_;
+    bool newVisible = false;
+
+    auto *ui = d->ui_;
+    if (ui && ui->addonInfo() &&
+        ui->addonInfo()->uiType() == UIType::OnScreenKeyboard) {
+        auto *vkui = static_cast<VirtualKeyboardUserInterface *>(ui);
+        newVisible = vkui->isVirtualKeyboardVisible();
+    }
+
+    if (oldVisible != newVisible) {
+        d->isVirtualKeyboardVisible_ = newVisible;
+
+        if (d->addonManager_->instance()) {
+            d->addonManager_->instance()->postEvent(
+                VirtualKeyboardVisibilityChangedEvent());
+        }
+    }
+}
+
 } // namespace fcitx
