@@ -20,6 +20,10 @@ using GObjectUniquePtr = UniqueCPtr<T, g_object_unref>;
 FCITX_DECLARE_LOG_CATEGORY(classicui_logcategory);
 #define CLASSICUI_DEBUG()                                                      \
     FCITX_LOGC(::fcitx::classicui::classicui_logcategory, Debug)
+#define CLASSICUI_ERROR()                                                      \
+    FCITX_LOGC(::fcitx::classicui::classicui_logcategory, Error)
+#define CLASSICUI_INFO()                                                       \
+    FCITX_LOGC(::fcitx::classicui::classicui_logcategory, Info)
 
 } // namespace classicui
 } // namespace fcitx
